@@ -11,7 +11,7 @@ var PATTERN_SEP = ';',
     DIGIT       = '#';
 
 /**
- * main funciton for parser
+ * main function for parser
  * @param str {string} pattern to be parsed (e.g. #,##0.###).
  */
 function parsePattern(pattern) {
@@ -56,7 +56,7 @@ function parsePattern(pattern) {
     p.negSuf = negative.substr(pos + trunkLen).replace(/\'/g, '');
   } else {
     // hardcoded '-' sign is fine as all locale use '-' as MINUS_SIGN. (\u2212 is the same as '-')
-    p.negPre = p.posPre + '-';
+    p.negPre = '-' + p.posPre;
     p.negSuf = p.posSuf;
   }
 
