@@ -9,9 +9,9 @@ describe('Binder', function() {
   }
 
   beforeEach(function() {
-    this.compileToHtml = function (content) {
+    this.compileToHtml = function(content) {
       var html;
-      inject(function($rootScope, $compile){
+      inject(function($rootScope, $compile) {
         content = jqLite(content);
         $compile(content)($rootScope);
         html = sortedHtml(content);
@@ -167,32 +167,8 @@ describe('Binder', function() {
     expect(element[0].childNodes.length).toEqual(1);
   }));
 
-  it('IfTextBindingThrowsErrorDecorateTheSpan', function() {
-    module(function($exceptionHandlerProvider){
-      $exceptionHandlerProvider.mode('log');
-    });
-    inject(function($rootScope, $exceptionHandler, $compile) {
-      element = $compile('<div>{{error.throw()}}</div>', null, true)($rootScope);
-      var errorLogs = $exceptionHandler.errors;
-
-      $rootScope.error = {
-        'throw': function() {throw 'ErrorMsg1';}
-      };
-      $rootScope.$apply();
-
-      $rootScope.error['throw'] = function() {throw 'MyError';};
-      errorLogs.length = 0;
-      $rootScope.$apply();
-      expect(errorLogs.shift().message).toMatch(/^\[\$interpolate:interr\] Can't interpolate: \{\{error.throw\(\)\}\}\nMyError/);
-
-      $rootScope.error['throw'] = function() {return 'ok';};
-      $rootScope.$apply();
-      expect(errorLogs.length).toBe(0);
-    });
-  });
-
   it('IfAttrBindingThrowsErrorDecorateTheAttribute', function() {
-    module(function($exceptionHandlerProvider){
+    module(function($exceptionHandlerProvider) {
       $exceptionHandlerProvider.mode('log');
     });
     inject(function($rootScope, $exceptionHandler, $compile) {
@@ -210,7 +186,7 @@ describe('Binder', function() {
 
       $rootScope.error['throw'] =  function() { return 'X';};
       $rootScope.$apply();
-      expect(errorLogs.length).toMatch(0);
+      expect(errorLogs.length).toMatch('0');
     });
   });
 
@@ -226,21 +202,21 @@ describe('Binder', function() {
     $rootScope.$apply();
 
     expect(sortedHtml(element)).toBe(
-        '<div>'+
+        '<div>' +
           '<!-- ngRepeat: m in model -->' +
-          '<div name="a" ng-repeat="m in model">'+
+          '<div name="a" ng-repeat="m in model">' +
             '<!-- ngRepeat: i in m.item -->' +
-            '<ul name="a1" ng-repeat="i in m.item"></ul>'+
+            '<ul name="a1" ng-repeat="i in m.item"></ul>' +
             '<!-- end ngRepeat: i in m.item -->' +
-            '<ul name="a2" ng-repeat="i in m.item"></ul>'+
+            '<ul name="a2" ng-repeat="i in m.item"></ul>' +
             '<!-- end ngRepeat: i in m.item -->' +
-          '</div>'+
+          '</div>' +
           '<!-- end ngRepeat: m in model -->' +
-          '<div name="b" ng-repeat="m in model">'+
+          '<div name="b" ng-repeat="m in model">' +
             '<!-- ngRepeat: i in m.item -->' +
-            '<ul name="b1" ng-repeat="i in m.item"></ul>'+
+            '<ul name="b1" ng-repeat="i in m.item"></ul>' +
             '<!-- end ngRepeat: i in m.item -->' +
-            '<ul name="b2" ng-repeat="i in m.item"></ul>'+
+            '<ul name="b2" ng-repeat="i in m.item"></ul>' +
             '<!-- end ngRepeat: i in m.item -->' +
           '</div>' +
           '<!-- end ngRepeat: m in model -->' +
@@ -272,6 +248,16 @@ describe('Binder', function() {
     $rootScope.hidden = 'false';
     $rootScope.$apply();
 
+    assertHidden(element);
+
+    $rootScope.hidden = 0;
+    $rootScope.$apply();
+
+    assertVisible(element);
+
+    $rootScope.hidden = false;
+    $rootScope.$apply();
+
     assertVisible(element);
 
     $rootScope.hidden = '';
@@ -291,6 +277,16 @@ describe('Binder', function() {
     $rootScope.show = 'false';
     $rootScope.$apply();
 
+    assertVisible(element);
+
+    $rootScope.show = false;
+    $rootScope.$apply();
+
+    assertHidden(element);
+
+    $rootScope.show = false;
+    $rootScope.$apply();
+
     assertHidden(element);
 
     $rootScope.show = '';
@@ -348,7 +344,7 @@ describe('Binder', function() {
   }));
 
   it('ActionOnAHrefThrowsError', function() {
-    module(function($exceptionHandlerProvider){
+    module(function($exceptionHandlerProvider) {
       $exceptionHandlerProvider.mode('log');
     });
     inject(function($rootScope, $exceptionHandler, $compile) {
@@ -373,7 +369,7 @@ describe('Binder', function() {
     expect(element.text()).toBe('123{{a}}{{b}}{{c}}');
   }));
 
-  it('ShouldTemplateBindPreElements', inject(function ($rootScope, $compile) {
+  it('ShouldTemplateBindPreElements', inject(function($rootScope, $compile) {
     element = $compile('<pre>Hello {{name}}!</pre>')($rootScope);
     $rootScope.name = "World";
     $rootScope.$apply();
@@ -446,7 +442,9 @@ describe('Binder', function() {
   it('ItShouldFireChangeListenersBeforeUpdate', inject(function($rootScope, $compile) {
     element = $compile('<div ng-bind="name"></div>')($rootScope);
     $rootScope.name = '';
-    $rootScope.$watch('watched', 'name=123');
+    $rootScope.$watch('watched', function() {
+      $rootScope.name = 123;
+    });
     $rootScope.watched = 'change';
     $rootScope.$apply();
     expect($rootScope.name).toBe(123);
