@@ -1,8 +1,30 @@
 'use strict';
 
-function MockWindow() {
+/* global getHash:true, stripHash:true */
+
+var historyEntriesLength;
+var sniffer = {};
+
+function MockWindow(options) {
+  if (typeof options !== 'object') {
+    options = {};
+  }
   var events = {};
   var timeouts = this.timeouts = [];
+  var locationHref = 'http://server/';
+  var committedHref = 'http://server/';
+  var mockWindow = this;
+  var msie = options.msie;
+  var ieState;
+
+  historyEntriesLength = 1;
+
+  function replaceHash(href, hash) {
+    // replace the hash with the new one (stripping off a leading hash if there is one)
+    // See hash setter spec: https://url.spec.whatwg.org/#urlutils-and-urlutilsreadonly-members
+    return stripHash(href) + '#' + hash.replace(/^#/,'');
+  }
+
 
   this.setTimeout = function(fn) {
     return timeouts.push(fn) - 1;
@@ -18,32 +40,83 @@ function MockWindow() {
   };
 
   this.addEventListener = function(name, listener) {
-    if (isUndefined(events[name])) events[name] = [];
+    if (angular.isUndefined(events[name])) events[name] = [];
     events[name].push(listener);
   };
 
-  this.attachEvent = function(name, listener) {
-    this.addEventListener(name.substr(2), listener);
-  };
-
   this.removeEventListener = noop;
-  this.detachEvent = noop;
 
   this.fire = function(name) {
     forEach(events[name], function(fn) {
-      fn({type: name}); // type to make jQuery happy
+      // type/target to make jQuery happy
+      fn({
+        type: name,
+        target: {
+          nodeType: 1
+        }
+      });
     });
   };
 
   this.location = {
-    href: 'http://server/',
-    replace: noop
+    get href() {
+      return committedHref;
+    },
+    set href(value) {
+      locationHref = value;
+      mockWindow.history.state = null;
+      historyEntriesLength++;
+      if (!options.updateAsync) this.flushHref();
+    },
+    get hash() {
+      return getHash(committedHref);
+    },
+    set hash(value) {
+      locationHref = replaceHash(locationHref, value);
+      if (!options.updateAsync) this.flushHref();
+    },
+    replace: function(url) {
+      locationHref = url;
+      mockWindow.history.state = null;
+      if (!options.updateAsync) this.flushHref();
+    },
+    flushHref: function() {
+      committedHref = locationHref;
+    }
   };
 
   this.history = {
-    replaceState: noop,
-    pushState: noop
+    pushState: function() {
+      this.replaceState.apply(this, arguments);
+      historyEntriesLength++;
+    },
+    replaceState: function(state, title, url) {
+      locationHref = url;
+      if (!options.updateAsync) committedHref = locationHref;
+      mockWindow.history.state = copy(state);
+      if (!options.updateAsync) this.flushHref();
+    },
+    flushHref: function() {
+      committedHref = locationHref;
+    }
   };
+  // IE 10-11 deserialize history.state on each read making subsequent reads
+  // different object.
+  if (!msie) {
+    this.history.state = null;
+  } else {
+    ieState = null;
+    Object.defineProperty(this.history, 'state', {
+      get: function() {
+        return copy(ieState);
+      },
+      set: function(value) {
+        ieState = value;
+      },
+      configurable: true,
+      enumerable: true
+    });
+  }
 }
 
 function MockDocument() {
@@ -55,7 +128,7 @@ function MockDocument() {
   this.find = function(name) {
     if (name == 'base') {
       return {
-        attr: function(name){
+        attr: function(name) {
           if (name == 'href') {
             return self.basePath;
           } else {
@@ -71,21 +144,18 @@ function MockDocument() {
 
 describe('browser', function() {
   /* global Browser: false */
-  var browser, fakeWindow, fakeDocument, logs, scripts, removedScripts, sniffer;
+  var browser, fakeWindow, fakeDocument, fakeLog, logs, scripts, removedScripts;
 
   beforeEach(function() {
     scripts = [];
     removedScripts = [];
-    sniffer = {history: true, hashchange: true};
+    sniffer = {history: true};
     fakeWindow = new MockWindow();
     fakeDocument = new MockDocument();
 
-    var fakeBody = [{appendChild: function(node){scripts.push(node);},
-                     removeChild: function(node){removedScripts.push(node);}}];
-
     logs = {log:[], warn:[], info:[], error:[]};
 
-    var fakeLog = {log: function() { logs.log.push(slice.call(arguments)); },
+    fakeLog = {log: function() { logs.log.push(slice.call(arguments)); },
                    warn: function() { logs.warn.push(slice.call(arguments)); },
                    info: function() { logs.info.push(slice.call(arguments)); },
                    error: function() { logs.error.push(slice.call(arguments)); }};
@@ -93,11 +163,58 @@ describe('browser', function() {
     browser = new Browser(fakeWindow, fakeDocument, fakeLog, sniffer);
   });
 
-  it('should contain cookie cruncher', function() {
-    expect(browser.cookies).toBeDefined();
+  describe('MockBrowser', function() {
+    describe('historyEntriesLength', function() {
+      it('should increment historyEntriesLength when setting location.href', function() {
+        expect(historyEntriesLength).toBe(1);
+        fakeWindow.location.href = '/foo';
+        expect(historyEntriesLength).toBe(2);
+      });
+
+      it('should not increment historyEntriesLength when using location.replace', function() {
+        expect(historyEntriesLength).toBe(1);
+        fakeWindow.location.replace('/foo');
+        expect(historyEntriesLength).toBe(1);
+      });
+
+      it('should increment historyEntriesLength when using history.pushState', function() {
+        expect(historyEntriesLength).toBe(1);
+        fakeWindow.history.pushState({a: 2}, 'foo', '/bar');
+        expect(historyEntriesLength).toBe(2);
+      });
+
+      it('should not increment historyEntriesLength when using history.replaceState', function() {
+        expect(historyEntriesLength).toBe(1);
+        fakeWindow.history.replaceState({a: 2}, 'foo', '/bar');
+        expect(historyEntriesLength).toBe(1);
+      });
+    });
+
+    describe('in IE', runTests({msie: true}));
+    describe('not in IE', runTests({msie: false}));
+
+    function runTests(options) {
+      return function() {
+        it('should return the same state object on every read', function() {
+          var msie = options.msie;
+
+          fakeWindow = new MockWindow({msie: msie});
+          fakeWindow.location.state = {prop: 'val'};
+          browser = new Browser(fakeWindow, fakeDocument, fakeLog, sniffer);
+
+          browser.url(fakeWindow.location.href, false, {prop: 'val'});
+          if (msie) {
+            expect(fakeWindow.history.state).not.toBe(fakeWindow.history.state);
+            expect(fakeWindow.history.state).toEqual(fakeWindow.history.state);
+          } else {
+            expect(fakeWindow.history.state).toBe(fakeWindow.history.state);
+          }
+        });
+      };
+    }
   });
 
-  describe('outstading requests', function() {
+  describe('outstanding requests', function() {
     it('should process callbacks immedietly with no outstanding requests', function() {
       var callback = jasmine.createSpy('callback');
       browser.notifyWhenNoOutstandingRequests(callback);
@@ -157,237 +274,6 @@ describe('browser', function() {
   });
 
 
-  describe('cookies', function() {
-
-    function deleteAllCookies() {
-      var cookies = document.cookie.split(";");
-      var path = location.pathname;
-
-      for (var i = 0; i < cookies.length; i++) {
-        var cookie = cookies[i];
-        var eqPos = cookie.indexOf("=");
-        var name = eqPos > -1 ? cookie.substr(0, eqPos) : cookie;
-        var parts = path.split('/');
-        while (parts.length) {
-          document.cookie = name + "=;path=" + (parts.join('/') || '/') + ";expires=Thu, 01 Jan 1970 00:00:00 GMT";
-          parts.pop();
-        }
-      }
-    }
-
-    beforeEach(function() {
-      deleteAllCookies();
-      expect(document.cookie).toEqual('');
-    });
-
-
-    afterEach(function() {
-      deleteAllCookies();
-      expect(document.cookie).toEqual('');
-    });
-
-
-    describe('remove all via (null)', function() {
-
-      it('should do nothing when no cookies are set', function() {
-        browser.cookies(null);
-        expect(document.cookie).toEqual('');
-        expect(browser.cookies()).toEqual({});
-      });
-
-    });
-
-    describe('remove via cookies(cookieName, undefined)', function() {
-
-      it('should remove a cookie when it is present', function() {
-        document.cookie = 'foo=bar;path=/';
-
-        browser.cookies('foo', undefined);
-
-        expect(document.cookie).toEqual('');
-        expect(browser.cookies()).toEqual({});
-      });
-
-
-      it('should do nothing when an nonexisting cookie is being removed', function() {
-        browser.cookies('doesntexist', undefined);
-        expect(document.cookie).toEqual('');
-        expect(browser.cookies()).toEqual({});
-      });
-    });
-
-
-    describe('put via cookies(cookieName, string)', function() {
-
-      it('should create and store a cookie', function() {
-        browser.cookies('cookieName', 'cookie=Value');
-        expect(document.cookie).toMatch(/cookieName=cookie%3DValue;? ?/);
-        expect(browser.cookies()).toEqual({'cookieName':'cookie=Value'});
-      });
-
-
-      it('should overwrite an existing unsynced cookie', function() {
-        document.cookie = "cookie=new;path=/";
-
-        var oldVal = browser.cookies('cookie', 'newer');
-
-        expect(document.cookie).toEqual('cookie=newer');
-        expect(browser.cookies()).toEqual({'cookie':'newer'});
-        expect(oldVal).not.toBeDefined();
-      });
-
-      it('should escape both name and value', function() {
-        browser.cookies('cookie1=', 'val;ue');
-        browser.cookies('cookie2=bar;baz', 'val=ue');
-
-        var rawCookies = document.cookie.split("; "); //order is not guaranteed, so we need to parse
-        expect(rawCookies.length).toEqual(2);
-        expect(rawCookies).toContain('cookie1%3D=val%3Bue');
-        expect(rawCookies).toContain('cookie2%3Dbar%3Bbaz=val%3Due');
-      });
-
-      it('should log warnings when 4kb per cookie storage limit is reached', function() {
-        var i, longVal = '', cookieStr;
-
-        for(i=0; i<4083; i++) {
-          longVal += '+';
-        }
-
-        cookieStr = document.cookie;
-        browser.cookies('x', longVal); //total size 4093-4096, so it should go through
-        expect(document.cookie).not.toEqual(cookieStr);
-        expect(browser.cookies()['x']).toEqual(longVal);
-        expect(logs.warn).toEqual([]);
-
-        browser.cookies('x', longVal + 'xxxx'); //total size 4097-4099, a warning should be logged
-        expect(logs.warn).toEqual(
-          [[ "Cookie 'x' possibly not set or overflowed because it was too large (4097 > 4096 " +
-             "bytes)!" ]]);
-
-        //force browser to dropped a cookie and make sure that the cache is not out of sync
-        browser.cookies('x', 'shortVal');
-        expect(browser.cookies().x).toEqual('shortVal'); //needed to prime the cache
-        cookieStr = document.cookie;
-        browser.cookies('x', longVal + longVal + longVal); //should be too long for all browsers
-
-        if (document.cookie !== cookieStr) {
-          this.fail(new Error("browser didn't drop long cookie when it was expected. make the " +
-              "cookie in this test longer"));
-        }
-
-        expect(browser.cookies().x).toEqual('shortVal');
-      });
-    });
-
-    describe('put via cookies(cookieName, string), if no <base href> ', function () {
-      beforeEach(function () {
-        fakeDocument.basePath = undefined;
-      });
-
-      it('should default path in cookie to "" (empty string)', function () {
-        browser.cookies('cookie', 'bender');
-        // This only fails in Safari and IE when cookiePath returns undefined
-        // Where it now succeeds since baseHref return '' instead of undefined
-        expect(document.cookie).toEqual('cookie=bender');
-      });
-    });
-
-    describe('get via cookies()[cookieName]', function() {
-
-      it('should return undefined for nonexistent cookie', function() {
-        expect(browser.cookies().nonexistent).not.toBeDefined();
-      });
-
-
-      it ('should return a value for an existing cookie', function() {
-        document.cookie = "foo=bar=baz;path=/";
-        expect(browser.cookies().foo).toEqual('bar=baz');
-      });
-
-      it('should return the the first value provided for a cookie', function() {
-        // For a cookie that has different values that differ by path, the
-        // value for the most specific path appears first.  browser.cookies()
-        // should provide that value for the cookie.
-        document.cookie = 'foo="first"; foo="second"';
-        expect(browser.cookies()['foo']).toBe('"first"');
-      });
-
-      it ('should unescape cookie values that were escaped by puts', function() {
-        document.cookie = "cookie2%3Dbar%3Bbaz=val%3Due;path=/";
-        expect(browser.cookies()['cookie2=bar;baz']).toEqual('val=ue');
-      });
-
-
-      it('should preserve leading & trailing spaces in names and values', function() {
-        browser.cookies(' cookie name ', ' cookie value ');
-        expect(browser.cookies()[' cookie name ']).toEqual(' cookie value ');
-        expect(browser.cookies()['cookie name']).not.toBeDefined();
-      });
-    });
-
-
-    describe('getAll via cookies()', function() {
-
-      it('should return cookies as hash', function() {
-        document.cookie = "foo1=bar1;path=/";
-        document.cookie = "foo2=bar2;path=/";
-        expect(browser.cookies()).toEqual({'foo1':'bar1', 'foo2':'bar2'});
-      });
-
-
-      it('should return empty hash if no cookies exist', function() {
-        expect(browser.cookies()).toEqual({});
-      });
-    });
-
-
-    it('should pick up external changes made to browser cookies', function() {
-      browser.cookies('oatmealCookie', 'drool');
-      expect(browser.cookies()).toEqual({'oatmealCookie':'drool'});
-
-      document.cookie = 'oatmealCookie=changed;path=/';
-      expect(browser.cookies().oatmealCookie).toEqual('changed');
-    });
-
-
-    it('should initialize cookie cache with existing cookies', function() {
-      document.cookie = "existingCookie=existingValue;path=/";
-      expect(browser.cookies()).toEqual({'existingCookie':'existingValue'});
-    });
-
-  });
-
-  describe('poller', function() {
-
-    it('should call functions in pollFns in regular intervals', function() {
-      var log = '';
-      browser.addPollFn(function() {log+='a';});
-      browser.addPollFn(function() {log+='b';});
-      expect(log).toEqual('');
-      fakeWindow.setTimeout.flush();
-      expect(log).toEqual('ab');
-      fakeWindow.setTimeout.flush();
-      expect(log).toEqual('abab');
-    });
-
-    it('should startPoller', function() {
-      expect(fakeWindow.timeouts.length).toEqual(0);
-
-      browser.addPollFn(function() {});
-      expect(fakeWindow.timeouts.length).toEqual(1);
-
-      //should remain 1 as it is the check fn
-      browser.addPollFn(function() {});
-      expect(fakeWindow.timeouts.length).toEqual(1);
-    });
-
-    it('should return fn that was passed into addPollFn', function() {
-      var fn = function() { return 1; };
-      var returnedFn = browser.addPollFn(fn);
-      expect(returnedFn).toBe(fn);
-    });
-  });
-
   describe('url', function() {
     var pushState, replaceState, locationReplace;
 
@@ -410,7 +296,7 @@ describe('browser', function() {
       browser.url('http://new.org');
 
       expect(pushState).toHaveBeenCalledOnce();
-      expect(pushState.argsForCall[0][2]).toEqual('http://new.org');
+      expect(pushState.calls.argsFor(0)[2]).toEqual('http://new.org');
 
       expect(replaceState).not.toHaveBeenCalled();
       expect(locationReplace).not.toHaveBeenCalled();
@@ -422,7 +308,7 @@ describe('browser', function() {
       browser.url('http://new.org', true);
 
       expect(replaceState).toHaveBeenCalledOnce();
-      expect(replaceState.argsForCall[0][2]).toEqual('http://new.org');
+      expect(replaceState.calls.argsFor(0)[2]).toEqual('http://new.org');
 
       expect(pushState).not.toHaveBeenCalled();
       expect(locationReplace).not.toHaveBeenCalled();
@@ -451,6 +337,17 @@ describe('browser', function() {
       expect(locationReplace).not.toHaveBeenCalled();
     });
 
+    it("should retain the # character when the only change is clearing the hash fragment, to prevent page reload", function() {
+      sniffer.history = true;
+
+      browser.url('http://server/#123');
+      expect(fakeWindow.location.href).toEqual('http://server/#123');
+
+      browser.url('http://server/');
+      expect(fakeWindow.location.href).toEqual('http://server/#');
+
+    });
+
     it('should use location.replace when history.replaceState not available', function() {
       sniffer.history = false;
       browser.url('http://new.org', true);
@@ -462,6 +359,7 @@ describe('browser', function() {
       expect(fakeWindow.location.href).toEqual('http://server/');
     });
 
+
     it('should use location.replace and not use replaceState when the url only changed in the hash fragment to please IE10/11', function() {
       sniffer.history = true;
       browser.url('http://server/#123', true);
@@ -473,10 +371,17 @@ describe('browser', function() {
       expect(fakeWindow.location.href).toEqual('http://server/');
     });
 
+
     it('should return $browser to allow chaining', function() {
       expect(browser.url('http://any.com')).toBe(browser);
     });
 
+    it('should return $browser to allow chaining even if the previous and current URLs and states match', function() {
+      expect(browser.url('http://any.com').url('http://any.com')).toBe(browser);
+      var state = { any: 'foo' };
+      expect(browser.url('http://any.com', false, state).url('http://any.com', false, state)).toBe(browser);
+      expect(browser.url('http://any.com', true, state).url('http://any.com', true, state)).toBe(browser);
+    });
 
     it('should decode single quotes to work around FF bug 407273', function() {
       fakeWindow.location.href = "http://ff-bug/?single%27quote";
@@ -504,7 +409,7 @@ describe('browser', function() {
       expect(fakeWindow.location.href).toBe('http://server/someOtherUrl');
     });
 
-    it('assumes that changes to location.hash occur in sync', function() {
+    it('assumes that changes to location.hash occur in sync', function(done) {
       // This is an asynchronous integration test that changes the
       // hash in all possible ways and checks
       // - whether the change to the hash can be read out in sync
@@ -513,7 +418,8 @@ describe('browser', function() {
           $realWin = jqLite(realWin),
           hashInHashChangeEvent = [];
 
-      runs(function() {
+      var job = createAsync(done);
+      job.runs(function() {
         $realWin.on('hashchange', hashListener);
 
         realWin.location.hash = '1';
@@ -522,17 +428,18 @@ describe('browser', function() {
         realWin.location.assign(realWin.location.href + '4');
 
         expect(realWin.location.hash).toBe('#1234');
-      });
-      waitsFor(function() {
+      })
+      .waitsFor(function() {
         return hashInHashChangeEvent.length > 3;
-      });
-      runs(function() {
+      })
+      .runs(function() {
         $realWin.off('hashchange', hashListener);
 
         forEach(hashInHashChangeEvent, function(hash) {
           expect(hash).toBe('#1234');
         });
-      });
+      }).done();
+      job.start();
 
       function hashListener() {
         hashInHashChangeEvent.push(realWin.location.hash);
@@ -541,6 +448,197 @@ describe('browser', function() {
 
   });
 
+  describe('url (with ie 11 weirdnesses)', function() {
+
+    it('url() should actually set the url, even if IE 11 is weird and replaces HTML entities in the URL', function() {
+      // this test can not be expressed with the Jasmine spies in the previous describe block, because $browser.url()
+      // needs to observe the change to location.href during its invocation to enter the failing code path, but the spies
+      // are not callThrough
+
+      sniffer.history = true;
+      var originalReplace = fakeWindow.location.replace;
+      fakeWindow.location.replace = function(url) {
+        url = url.replace('&not', '¬');
+        // I really don't know why IE 11 (sometimes) does this, but I am not the only one to notice:
+        // https://connect.microsoft.com/IE/feedback/details/1040980/bug-in-ie-which-interprets-document-location-href-as-html
+        originalReplace.call(this, url);
+      };
+
+      // the initial URL contains a lengthy oauth token in the hash
+      var initialUrl = 'http://test.com/oauthcallback#state=xxx%3D&not-before-policy=0';
+      fakeWindow.location.href = initialUrl;
+      browser = new Browser(fakeWindow, fakeDocument, fakeLog, sniffer);
+
+      // somehow, $location gets a version of this url where the = is no longer escaped, and tells the browser:
+      var initialUrlFixedByLocation = initialUrl.replace('%3D', '=');
+      browser.url(initialUrlFixedByLocation, true, null);
+      expect(browser.url()).toEqual(initialUrlFixedByLocation);
+
+      // a little later (but in the same digest cycle) the view asks $location to replace the url, which tells $browser
+      var secondUrl = 'http://test.com/otherView';
+      browser.url(secondUrl, true, null);
+      expect(browser.url()).toEqual(secondUrl);
+    });
+
+  });
+
+  describe('url (when state passed)', function() {
+    var currentHref, pushState, replaceState, locationReplace;
+
+    beforeEach(function() {
+    });
+
+    describe('in IE', runTests({msie: true}));
+    describe('not in IE', runTests({msie: false}));
+
+    function runTests(options) {
+      return function() {
+        beforeEach(function() {
+          sniffer = {history: true};
+
+          fakeWindow = new MockWindow({msie: options.msie});
+          currentHref = fakeWindow.location.href;
+          pushState = spyOn(fakeWindow.history, 'pushState').and.callThrough();
+          replaceState = spyOn(fakeWindow.history, 'replaceState').and.callThrough();
+          locationReplace = spyOn(fakeWindow.location, 'replace').and.callThrough();
+
+          browser = new Browser(fakeWindow, fakeDocument, fakeLog, sniffer);
+          browser.onUrlChange(function() {});
+        });
+
+        it('should change state', function() {
+          browser.url(currentHref, false, {prop: 'val1'});
+          expect(fakeWindow.history.state).toEqual({prop: 'val1'});
+          browser.url(currentHref + '/something', false, {prop: 'val2'});
+          expect(fakeWindow.history.state).toEqual({prop: 'val2'});
+        });
+
+        it('should allow to set falsy states (except `undefined`)', function() {
+          fakeWindow.history.state = {prop: 'val1'};
+          fakeWindow.fire('popstate');
+
+          browser.url(currentHref, false, null);
+          expect(fakeWindow.history.state).toBe(null);
+
+          browser.url(currentHref, false, false);
+          expect(fakeWindow.history.state).toBe(false);
+
+          browser.url(currentHref, false, '');
+          expect(fakeWindow.history.state).toBe('');
+
+          browser.url(currentHref, false, 0);
+          expect(fakeWindow.history.state).toBe(0);
+        });
+
+        it('should treat `undefined` state as `null`', function() {
+          fakeWindow.history.state = {prop: 'val1'};
+          fakeWindow.fire('popstate');
+
+          browser.url(currentHref, false, undefined);
+          expect(fakeWindow.history.state).toBe(null);
+        });
+
+        it('should do pushState with the same URL and a different state', function() {
+          browser.url(currentHref, false, {prop: 'val1'});
+          expect(fakeWindow.history.state).toEqual({prop: 'val1'});
+
+          browser.url(currentHref, false, null);
+          expect(fakeWindow.history.state).toBe(null);
+
+          browser.url(currentHref, false, {prop: 'val2'});
+          browser.url(currentHref, false, {prop: 'val3'});
+          expect(fakeWindow.history.state).toEqual({prop: 'val3'});
+        });
+
+        it('should do pushState with the same URL and deep equal but referentially different state', function() {
+          fakeWindow.history.state = {prop: 'val'};
+          fakeWindow.fire('popstate');
+          expect(historyEntriesLength).toBe(1);
+
+          browser.url(currentHref, false, {prop: 'val'});
+          expect(fakeWindow.history.state).toEqual({prop: 'val'});
+          expect(historyEntriesLength).toBe(2);
+        });
+
+        it('should not do pushState with the same URL and state from $browser.state()', function() {
+          browser.url(currentHref, false, {prop: 'val'});
+
+          pushState.calls.reset();
+          replaceState.calls.reset();
+          locationReplace.calls.reset();
+
+          browser.url(currentHref, false, browser.state());
+          expect(pushState).not.toHaveBeenCalled();
+          expect(replaceState).not.toHaveBeenCalled();
+          expect(locationReplace).not.toHaveBeenCalled();
+        });
+      };
+    }
+  });
+
+  describe('state', function() {
+    var currentHref;
+
+    beforeEach(function() {
+      sniffer = {history: true};
+      currentHref = fakeWindow.location.href;
+    });
+
+    it('should not access `history.state` when `$sniffer.history` is false', function() {
+      // In the context of a Chrome Packaged App, although `history.state` is present, accessing it
+      // is not allowed and logs an error in the console. We should not try to access
+      // `history.state` in contexts where `$sniffer.history` is false.
+
+      var historyStateAccessed = false;
+      var mockSniffer = {history: false};
+      var mockWindow = new MockWindow();
+
+      var _state = mockWindow.history.state;
+      Object.defineProperty(mockWindow.history, 'state', {
+        get: function() {
+          historyStateAccessed = true;
+          return _state;
+        }
+      });
+
+      var browser = new Browser(mockWindow, fakeDocument, fakeLog, mockSniffer);
+
+      expect(historyStateAccessed).toBe(false);
+    });
+
+    describe('in IE', runTests({msie: true}));
+    describe('not in IE', runTests({msie: false}));
+
+
+    function runTests(options) {
+      return function() {
+        beforeEach(function() {
+          fakeWindow = new MockWindow({msie: options.msie});
+          browser = new Browser(fakeWindow, fakeDocument, fakeLog, sniffer);
+        });
+
+        it('should return history.state', function() {
+          browser.url(currentHref, false, {prop: 'val'});
+          expect(browser.state()).toEqual({prop: 'val'});
+          browser.url(currentHref, false, 2);
+          expect(browser.state()).toEqual(2);
+          browser.url(currentHref, false, null);
+          expect(browser.state()).toEqual(null);
+        });
+
+        it('should return null if history.state is undefined', function() {
+          browser.url(currentHref, false, undefined);
+          expect(browser.state()).toBe(null);
+        });
+
+        it('should return the same state object in subsequent invocations in IE', function() {
+          browser.url(currentHref, false, {prop: 'val'});
+          expect(browser.state()).toBe(browser.state());
+        });
+      };
+    }
+  });
+
   describe('urlChange', function() {
     var callback;
 
@@ -549,7 +647,7 @@ describe('browser', function() {
     });
 
     afterEach(function() {
-      if (!jQuery) jqLite(fakeWindow).dealoc();
+      if (!jQuery) jqLiteDealoc(fakeWindow);
     });
 
     it('should return registered callback', function() {
@@ -562,97 +660,94 @@ describe('browser', function() {
       fakeWindow.location.href = 'http://server/new';
 
       fakeWindow.fire('popstate');
-      expect(callback).toHaveBeenCalledWith('http://server/new');
+      expect(callback).toHaveBeenCalledWith('http://server/new', null);
 
       fakeWindow.fire('hashchange');
       fakeWindow.setTimeout.flush();
       expect(callback).toHaveBeenCalledOnce();
     });
 
-    it('should forward only popstate event when both history and hashchange supported', function() {
+    it('should forward only popstate event when history supported', function() {
       sniffer.history = true;
-      sniffer.hashchange = true;
       browser.onUrlChange(callback);
       fakeWindow.location.href = 'http://server/new';
 
       fakeWindow.fire('popstate');
-      expect(callback).toHaveBeenCalledWith('http://server/new');
+      expect(callback).toHaveBeenCalledWith('http://server/new', null);
 
       fakeWindow.fire('hashchange');
       fakeWindow.setTimeout.flush();
       expect(callback).toHaveBeenCalledOnce();
     });
 
-    it('should forward hashchange event with new url when only hashchange supported', function() {
+    it('should forward hashchange event with new url when history not supported', function() {
       sniffer.history = false;
-      sniffer.hashchange = true;
       browser.onUrlChange(callback);
       fakeWindow.location.href = 'http://server/new';
 
       fakeWindow.fire('hashchange');
-      expect(callback).toHaveBeenCalledWith('http://server/new');
+      expect(callback).toHaveBeenCalledWith('http://server/new', null);
 
       fakeWindow.fire('popstate');
       fakeWindow.setTimeout.flush();
       expect(callback).toHaveBeenCalledOnce();
     });
 
-    it('should use polling when neither history nor hashchange supported', function() {
+    it('should not fire urlChange if changed by browser.url method', function() {
       sniffer.history = false;
-      sniffer.hashchange = false;
       browser.onUrlChange(callback);
+      browser.url('http://new.com');
 
-      fakeWindow.location.href = 'http://server.new';
-      fakeWindow.setTimeout.flush();
-      expect(callback).toHaveBeenCalledWith('http://server.new');
-
-      callback.reset();
-
-      fakeWindow.fire('popstate');
       fakeWindow.fire('hashchange');
       expect(callback).not.toHaveBeenCalled();
     });
 
-    describe('after an initial location change by browser.url method when neither history nor hashchange supported', function() {
+    describe('state handling', function() {
+      var currentHref;
+
       beforeEach(function() {
-        sniffer.history = false;
-        sniffer.hashchange = false;
-        browser.url("http://server/#current");
+        sniffer = {history: true};
+        currentHref = fakeWindow.location.href;
       });
 
-      it('should fire callback with the correct URL on location change outside of angular', function() {
-        browser.onUrlChange(callback);
+      describe('in IE', runTests({msie: true}));
+      describe('not in IE', runTests({msie: false}));
 
-        fakeWindow.location.href = 'http://server/#new';
-        fakeWindow.setTimeout.flush();
-        expect(callback).toHaveBeenCalledWith('http://server/#new');
+      function runTests(options) {
+        return function() {
+          beforeEach(function() {
+            fakeWindow = new MockWindow({msie: options.msie});
+            browser = new Browser(fakeWindow, fakeDocument, fakeLog, sniffer);
+          });
 
-        fakeWindow.fire('popstate');
-        fakeWindow.fire('hashchange');
-        expect(callback).toHaveBeenCalledOnce();
-      });
+          it('should fire onUrlChange listeners only once if both popstate and hashchange triggered', function() {
+            fakeWindow.history.state = {prop: 'val'};
+            browser.onUrlChange(callback);
 
+            fakeWindow.fire('hashchange');
+            fakeWindow.fire('popstate');
+            expect(callback).toHaveBeenCalledOnce();
+          });
+        };
+      }
     });
 
-    it('should not fire urlChange if changed by browser.url method (polling)', function() {
-      sniffer.history = false;
-      sniffer.hashchange = false;
-      browser.onUrlChange(callback);
-      browser.url('http://new.com');
 
+    it("should stop calling callbacks when application has been torn down", function() {
+      sniffer.history = true;
+      browser.onUrlChange(callback);
+      fakeWindow.location.href = 'http://server/new';
+
+      browser.$$applicationDestroyed();
+
+      fakeWindow.fire('popstate');
+      expect(callback).not.toHaveBeenCalled();
+
+      fakeWindow.fire('hashchange');
       fakeWindow.setTimeout.flush();
       expect(callback).not.toHaveBeenCalled();
     });
 
-    it('should not fire urlChange if changed by browser.url method (hashchange)', function() {
-      sniffer.history = false;
-      sniffer.hashchange = true;
-      browser.onUrlChange(callback);
-      browser.url('http://new.com');
-
-      fakeWindow.fire('hashchange');
-      expect(callback).not.toHaveBeenCalled();
-    });
   });
 
 
@@ -690,15 +785,18 @@ describe('browser', function() {
   describe('integration tests with $location', function() {
 
     function setup(options) {
+      fakeWindow = new MockWindow(options);
+      browser = new Browser(fakeWindow, fakeDocument, fakeLog, sniffer);
+
       module(function($provide, $locationProvider) {
-        spyOn(fakeWindow.history, 'pushState').andCallFake(function(stateObj, title, newUrl) {
+
+        spyOn(fakeWindow.history, 'pushState').and.callFake(function(stateObj, title, newUrl) {
           fakeWindow.location.href = newUrl;
         });
-        spyOn(fakeWindow.location, 'replace').andCallFake(function(newUrl) {
+        spyOn(fakeWindow.location, 'replace').and.callFake(function(newUrl) {
           fakeWindow.location.href = newUrl;
         });
         $provide.value('$browser', browser);
-        browser.pollFns = [];
 
         sniffer.history = options.history;
         $provide.value('$sniffer', sniffer);
@@ -707,7 +805,7 @@ describe('browser', function() {
       });
     }
 
-    describe('update $location when it was changed outside of Angular in sync '+
+    describe('update $location when it was changed outside of Angular in sync ' +
        'before $digest was called', function() {
 
       it('should work with no history support, no html5Mode', function() {
@@ -796,13 +894,13 @@ describe('browser', function() {
       fakeWindow.location.href = 'http://server/some/deep/path';
       var changeUrlCount = 0;
       var _url = browser.url;
-      browser.url = function(newUrl, replace) {
+      browser.url = function(newUrl, replace, state) {
         if (newUrl) {
           changeUrlCount++;
         }
         return _url.call(this, newUrl, replace);
       };
-      spyOn(browser, 'url').andCallThrough();
+      spyOn(browser, 'url').and.callThrough();
       inject(function($rootScope, $location) {
         $rootScope.$digest();
         $rootScope.$digest();
@@ -811,19 +909,42 @@ describe('browser', function() {
 
         // from $location for rewriting the initial url into a hash url
         expect(browser.url).toHaveBeenCalledWith('http://server/#/some/deep/path', true);
-        // from the initial call to the watch in $location for watching $location
-        expect(browser.url).toHaveBeenCalledWith('http://server/#/some/deep/path', false);
-        expect(changeUrlCount).toBe(2);
+        expect(changeUrlCount).toBe(1);
       });
 
     });
+
+    // issue #12241
+    it('should not infinite digest if the browser does not synchronously update the location properties', function() {
+      setup({
+        history: true,
+        html5Mode: true,
+        updateAsync: true // Simulate a browser that doesn't update the href synchronously
+      });
+
+      inject(function($location, $rootScope) {
+
+        // Change the hash within Angular and check that we don't infinitely digest
+        $location.hash('newHash');
+        expect(function() { $rootScope.$digest(); }).not.toThrow();
+        expect($location.absUrl()).toEqual('http://server/#newHash');
+
+        // Now change the hash from outside Angular and check that $location updates correctly
+        fakeWindow.location.hash = '#otherHash';
+
+        // simulate next tick - since this browser doesn't update synchronously
+        fakeWindow.location.flushHref();
+        fakeWindow.fire('hashchange');
+
+        expect($location.absUrl()).toEqual('http://server/#otherHash');
+      });
+    });
   });
 
   describe('integration test with $rootScope', function() {
 
     beforeEach(module(function($provide, $locationProvider) {
       $provide.value('$browser', browser);
-      browser.pollFns = [];
     }));
 
     it('should not interfere with legacy browser url replace behavior', function() {
@@ -831,11 +952,13 @@ describe('browser', function() {
         var current = fakeWindow.location.href;
         var newUrl = 'notyet';
         sniffer.history = false;
+        expect(historyEntriesLength).toBe(1);
         browser.url(newUrl, true);
         expect(browser.url()).toBe(newUrl);
+        expect(historyEntriesLength).toBe(1);
         $rootScope.$digest();
         expect(browser.url()).toBe(newUrl);
-        expect(fakeWindow.location.href).toBe(current);
+        expect(historyEntriesLength).toBe(1);
       });
     });
 
